/*
 * Copyright 2006 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.codegen.impl;

import java.util.Iterator;
import java.util.Map;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.seasar.codegen.ImportCodeData;
import org.seasar.codegen.element.DataType;
import org.seasar.codegen.element.Field;
import org.seasar.codegen.element.FieldSetting;
import org.seasar.codegen.element.LinkTable;
import org.seasar.codegen.element.PrimaryKey;
import org.seasar.codegen.element.Table;
import org.seasar.codegen.element.util.DataTypeSelectUtil;
import org.seasar.codegen.impl.util.CreateTableTypeToTypeUtil;
import org.seasar.codegen.impl.util.ExcelUtil;
import org.seasar.framework.util.StringUtil;

/**
 * Excel ̃e[u`ǂݍ݊NXB
 * 
 * @author glad
 */
public abstract class ExcelImportCodeDataSupport implements ImportCodeData {

    protected int attributeNameCol;
    protected int columnNameCol;
    protected int dataTypeCol;
    protected int notNullCol;
    protected int primaryKeyCol;
    protected int foreignKeyCol;
    protected int defaultValueCol;
    
    protected int relationTypeCol;
    protected int parentTableCol;
    protected int parentColumnCol;
    protected int cardinalityCol;
    protected int parentPropertyCol;
    protected int childPropertyCol;
    
    protected DataTypeSelectUtil dataTypeSelectUtil;

    public void setDataTypeSelectUtil(DataTypeSelectUtil dataTypeSelectUtil) {
        this.dataTypeSelectUtil = dataTypeSelectUtil;
    }
    
    protected Field getField(HSSFRow row) {
        String columnName = getString(row, columnNameCol);
        if (StringUtil.isEmpty(columnName)) {
            return null;
        }
        Field field = new Field();
        field.setFieldName(columnName);
        field.setDataType(getDataType(row));
        return field;
    }

    protected DataType getDataType(HSSFRow row) {
        String dataType = getString(row, dataTypeCol);
        String notNull  = getString(row, notNullCol);
        String defaultValue = getString(row, defaultValueCol);
        return getDataType(dataType, notNull, defaultValue);
    }

    protected DataType getDataType(
            String dataType, String notNull, String defaultValue) {
        FieldSetting fieldSetting = new FieldSetting();
        String typeName = CreateTableTypeToTypeUtil.getTypeName(dataType);
        String langType = dataTypeSelectUtil.convDBTypeToDataType(typeName);
        fieldSetting.setTypeName(langType);
        String length = CreateTableTypeToTypeUtil.getLength(dataType, typeName);
        int precision = CreateTableTypeToTypeUtil.getFullLength(length);
        fieldSetting.setColmnSize(precision);
        int scale = CreateTableTypeToTypeUtil.getPointNumberLength(length);
        fieldSetting.setPointNumber(scale);
        boolean isNotNull = "NOT NULL".equalsIgnoreCase(notNull);
        fieldSetting.setNotNull(isNotNull);
        fieldSetting.setFieldDefault(defaultValue);
        DataType type = dataTypeSelectUtil.selectBestDataType(fieldSetting);
        return type;
    }
    
    protected PrimaryKey getPrimaryKey(HSSFRow row, Field field) {
        String isPrimaryKey = getString(row, primaryKeyCol);
        if (!isTrue(isPrimaryKey)) {
            return null;
        }
        PrimaryKey primaryKey = new PrimaryKey();
        primaryKey.setField(field);
        return primaryKey;
    }
    
    protected LinkTable getParentLink(HSSFRow row, Field field) {
        String relationType = getString(row, relationTypeCol);
        if (StringUtil.isEmpty(relationType)) {
            return null;
        }
        LinkTable link = new LinkTable();
        link.setTableName(getString(row, parentTableCol));
        link.setParentFieldName(getString(row, parentColumnCol));
        link.setChildFieldName(field.getFieldName());
        return link;
    }

    protected void setupChildLinks(Map tableMap) {
        for (Iterator i = tableMap.values().iterator(); i.hasNext();) {
            setupChildLinks((Table)i.next(), tableMap);
        }
    }

    protected void setupChildLinks(Table child, Map tableMap) {
        for (Iterator i = child.getParentTable().iterator(); i.hasNext();) {
            LinkTable parentLink = (LinkTable)i.next();
            String tableName = parentLink.getTableName();
            Table parent = (Table)tableMap.get(tableName);
            LinkTable childLink = getChildLink(child, parentLink);
            parent.addChildTable(childLink);
        }
    }

    protected LinkTable getChildLink(Table child, LinkTable parentLink) {
        LinkTable link = new LinkTable();
        link.setTableName(child.getTableName());
        link.setParentFieldName(parentLink.getParentFieldName());
        link.setChildFieldName(parentLink.getChildFieldName());
        return link;
    }

    protected boolean isTrue(String s) {
        if (StringUtil.isEmpty(s)) {
            return false;
        }
        s = s.toLowerCase();
        return "yes".equals(s) || "true".equals(s);
    }
    
    protected String getString(HSSFSheet sheet, int row, int col) {
        return ExcelUtil.getString(sheet, row, col);
    }
    
    protected String getString(HSSFRow row, int col) {
        return ExcelUtil.getString(row, col);
    }
    
}
